module KakaxiParse
  class API
    BASE_URL = 'https://parseapi.back4app.com/'

    def initialize(endpoint, params: {})
      @uri = URI.parse(BASE_URL + endpoint)
      @params = params
    end

    def get
      @uri.query = URI.encode_www_form(@params)
      request = Net::HTTP::Get.new(@uri)
      set_auth_header!(request)
      request(request)
    end

    def put
      request = Net::HTTP::Put.new(@uri)
      request.body = @params.to_json
      set_auth_header!(request)
      request['Content-Type'] = 'application/json'
      request(request)
    end

    def post
      request = Net::HTTP::Post.new(@uri)
      request.body = @params.to_json
      set_auth_header!(request)
      request['Content-Type'] = 'application/json'
      request(request)
    end

    def delete
      request = Net::HTTP::Delete.new(@uri)
      set_auth_header!(request)
      request(request)
    end

    private
    def set_auth_header!(request)
      request['X-Parse-Application-Id'] = ENV['PARSE_ID']
      request['X-Parse-Master-Key'] = ENV['PARSE_MASTER_KEY']
    end

    def request(request)
      Net::HTTP.start(@uri.host, @uri.port, use_ssl: true) do |https|
        JSON.parse(https.request(request).body)
      end
    end
  end
end
