module KakaxiParse
  class Farm < BaseModel
    attr_accessor :farmer_id, :location
    @@belongs_to = [User]

    def initialize(
          id: nil,
          name: nil,
          farmer_id: nil,
          location:  nil,
          updated_at: nil,
          created_at: nil
        )
      @farmer_id = farmer_id
      @location  = location
      super(id, name, updated_at, created_at)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, hash, limit, @@belongs_to)) if valid_key?(hash)
    end

    def self.short_name
      'farm'
    end

    def save
      id ? false : super(self, { name: name, farmer_id: farmer_id }, @@belongs_to)
    end

    def update(params)
      original_params = build_original_params(params)
      if super(self, params, @@belongs_to)
        original_params.each { |key, value| self.send("#{key}=", value) }
        true
      else
        false
      end
    end

    def farmer
      User.find(farmer_id)
    end

    def devices
      KakaxiDevice.where(farm_id: id)
    end

    private
    def self.valid_key?(hash)
      hash.keys.all? { |key| %i(id name farmer_id updated_at created_at).include? key }
    end

    def self.to_objects(farms)
      farms.map do |farm|
        new(
          id: farm['objectId'],
          name: farm['name'],
          farmer_id: farm['farmer'] && farm['farmer']['objectId'],
          location:  farm['location'],
          updated_at: farm['updatedAt'],
          created_at: farm['createdAt']
        )
      end
    end
  end
end
