module KakaxiParse
  class FarmManagerUser < BaseModel
    def initialize(id: nil, devices: nil, updated_at: nil, created_at: nil)
      @devices = devices
      super(id, nil, updated_at, created_at)
    end

    def self.short_name
      'farm_manager_user'
    end

    def self.all(limit: DEFUALT_LIMIT)
      to_objects(API.new("classes/#{self.demodulize}", params: { limit: limit }).get)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, hash, limit)) if valid_key?(hash)
    end

    def save
      params = {}
      params = Datatype::AddRelation.new(KakaxiDevice, @devices.map(&:id)).to_hash if @devices
      id ? false : super(self, params)
    end

    def add_device(id)
      params = Datatype::AddRelation.new(KakaxiDevice, [id]).to_hash
      result = API.new("classes/#{ self.class.demodulize }/#{ self.id }", params: params).put
      if result.has_key? 'updatedAt'
        self.updated_at = DateTime.strptime(result['updatedAt'], '%Y-%m-%dT%H:%M:%S.%LZ')
        true
      else
        false
      end
    end

    def remove_device(id)
      params = Datatype::RemoveRelation.new(KakaxiDevice, [id]).to_hash
      result = API.new("classes/#{ self.class.demodulize }/#{ self.id }", params: params).put
      if result.has_key? 'updatedAt'
        self.updated_at = DateTime.strptime(result['updatedAt'], '%Y-%m-%dT%H:%M:%S.%LZ')
        true
      else
        false
      end
    end

    private
    def self.valid_key?(hash)
      hash.keys.all? { |key| %i(id devices updated_at created_at).include? key }
    end

    def self.to_objects(farm_manager_users)
      farm_manager_users.map do |user|
        new(
          id: user['objectId'],
          updated_at: user['updatedAt'],
          created_at: user['createdAt']
        )
      end
    end
  end
end
