# String id
# String name
# String url
# String device_id
# Datetime taken_at
# Datetime updated_at
# Datetime created_at

module KakaxiParse
  class IntervalPhoto < BaseModel
    attr_accessor :url, :taken_at, :device_id
    @@belongs_to = [KakaxiDevice]

    def initialize(
          id: nil,
          name: nil,
          url: nil,
          device_id: nil,
          taken_at: nil,
          updated_at: nil,
          created_at: nil
        )
      @url = url
      @device_id = Datatype::Pointer.new(KakaxiDevice, device_id)
      @taken_at = DateTime.strptime(taken_at, '%Y-%m-%dT%H:%M:%S.%LZ')
      super(id, name, updated_at, created_at)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, hash, limit, @@belongs_to)) if valid_key?(hash)
    end

    def self.short_name
      'photo'
    end

    def device_id
      @device_id.id
    end

    def device_id=(id)
      @device_id.id = id
    end

    private
    def self.valid_key?(hash)
      hash.keys.all? { |key| %i(id name url device_id taken_at updated_at created_at).include? key }
    end

    def self.to_objects(photos)
      photos.map do |photo|
        new(
          id: photo['objectId'],
          name: photo['photo'] && photo['photo']['name'],
          url: photo['photo'] && photo['photo']['url'],
          device_id: photo['device'] && photo['device']['objectId'],
          taken_at: photo['takenAt']['iso'],
          updated_at: photo['updatedAt'],
          created_at: photo['createdAt']
        )
      end
    end
  end
end

