module KakaxiParse
  class KakaxiDevice < BaseModel
    COLUMNS = %i(carrier_name iccid camera_finish_hour camera_start_hour imei
      camera_interval udid battery upload_start_hour beacon_minor scfg
      status avatar_type auto_post_interval beacon_major coordinates
      tzoffset camera_quality imsi tel serial_number farm_id farm_manager_user_id)
    attr_accessor *COLUMNS

    @@belongs_to = [Farm]
    @@related_to = [FarmManagerUser]

    def initialize(
          id: nil,
          name: nil,
          carrier_name: nil,
          iccid: nil,
          camera_finish_hour: nil,
          camera_start_hour: nil,
          imei: nil,
          camera_interval: nil,
          udid: nil,
          battery: nil,
          upload_start_hour: nil,
          beacon_minor: nil,
          scfg: nil,
          status: nil,
          avatar_type: nil,
          auto_post_interval: nil,
          beacon_major: nil,
          coordinates: nil,
          tzoffset: nil,
          camera_quality: nil,
          imsi: nil,
          tel: nil,
          serial_number: nil,
          farm_id: nil,
          updated_at: nil,
          created_at: nil
        )
       @carrier_name = carrier_name
       @iccid = iccid
       @camera_finish_hour = camera_finish_hour
       @camera_start_hour = camera_start_hour
       @imei = imei
       @camera_interval = camera_interval
       @udid = udid
       @battery = battery
       @upload_start_hour = upload_start_hour
       @beacon_minor = beacon_minor
       @scfg = scfg
       @status = status
       @avatar_type = avatar_type
       @auto_post_interval = auto_post_interval
       @beacon_major = beacon_major
       @coordinates = coordinates
       @tzoffset = tzoffset
       @camera_quality = camera_quality
       @imsi = imsi
       @tel = tel
       @serial_number = serial_number
       @farm_id = farm_id
       super(id, name, updated_at, created_at)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, validate_params(hash), limit, @@belongs_to, @@related_to)) if valid_key?(hash)
    end

    def self.short_name
      'device'
    end

    def self.reference_name
      'devices'
    end

    def farm
      Farm.find(farm_id: farm_id)
    end

    def save
      params = COLUMNS.push(:name).each_with_object({}) do |column, params|
        params[column] = self.send(column) unless self.send(column).nil?
      end
      id ? false : super(self, params, @@belongs_to)
    end

    def update(params)
      original_params = build_original_params(params)
      if super(self, KakaxiDevice.validate_params(params), @@belongs_to)
        original_params.each { |key, value| self.send("#{key}=", value) }
        true
      else
        false
      end
    end

    private
    def self.hour_to_time(params)
      %i(camera_finish_hour camera_start_hour upload_start_hour).each { |hour| BaseModel.replace_key(params, hour, hour.to_s.gsub(/hour/, 'time').to_sym) }
      params
    end

    def self.validate_params(params)
      params = hour_to_time(params)
      if params.has_key? :coordinates
        params = replace_key(params, :coordinates, :coordinate)
        params[:coordinate] = params[:coordinate].to_hash
      end
      params
    end

    def self.valid_key?(hash)
      hash.keys.all? { |key| COLUMNS.push(*%i(id name updated_at created_at)).include? key }
    end

    def self.to_objects(devices)
      devices.map do |device|
        new(
          id: device['objectId'],
          name: device['name'],
          updated_at: device['updatedAt'],
          created_at: device['createdAt'],
          farm_id: device['farm'] && device['farm']['objectId'],
          carrier_name: device['carrierName'],
          iccid: device['ICCID'],
          camera_finish_hour: device['cameraFinishTime'],
          camera_start_hour: device['cameraStartTime'],
          imei: device['IMEI'],
          camera_interval: device['cameraInterval'],
          udid: device['udid'],
          battery: device['battery'],
          upload_start_hour: device['uploadStartTime'],
          beacon_minor: device['beaconMinor'],
          scfg: device['scfg'],
          status: device['status'],
          avatar_type: device['avatarType'],
          auto_post_interval: device['autoPostInterval'],
          beacon_major: device['beaconMajor'],
          coordinates: device['coordinate'] && Datatype::GeoPoint.new(device['coordinate']['latitude'], device['coordinate']['longitude']),
          tzoffset: device['tzoffset'],
          camera_quality: device['cameraQuality'],
          imsi: device['IMSI'],
          tel: device['tel'],
          serial_number: device['serialNumber'] || device['serialnumber'],
        )
      end
    end
  end
end
