module KakaxiParse
  class Role < BaseModel
    def initialize(
          id: nil,
          name: nil,
          updated_at: nil,
          created_at: nil
        )
      super(id, name, updated_at, created_at)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, hash, limit)) if valid_key?(hash)
    end

    def self.name
      '_Role'
    end

    def self.short_name
      'role'
    end

    def add_user(id)
      params = Datatype::AddRelation.new(User, [id]).to_hash
      result = API.new("classes/#{self.class.demodulize}/#{ self.id }", params: params).put
      if result.has_key? 'updatedAt'
        self.updated_at = DateTime.strptime(result['updatedAt'], '%Y-%m-%dT%H:%M:%S.%LZ')
        true
      else
        false
      end
    end

    private
    def self.valid_key?(hash)
      hash.keys.all? { |key| %i(id name farmer_id updated_at created_at).include? key }
    end

    def self.to_objects(roles)
      roles.map do |role|
        new(
          id: role['objectId'],
          name: role['name'],
          updated_at: role['updatedAt'],
          created_at: role['createdAt']
        )
      end
    end
  end
end
