module KakaxiParse
  class Timelapse < BaseModel
    COLUMNS = [:start_datetime, :end_datetime, :thumbnail_url, :url, :device_id]
    attr_accessor *COLUMNS

    @@belongs_to = [KakaxiDevice]

    def initialize(
      id: nil,
      name: nil,
      created_at: nil,
      updated_at: nil,
      start_datetime: nil,
      end_datetime: nil,
      thumbnail_url: nil,
      url: nil,
      device_id: nil
      )

      @start_datetime = DateTime.iso8601(start_datetime)
      @end_datetime   = DateTime.iso8601(end_datetime)
      @thumbnail_url  = thumbnail_url
      @url            = url
      @device_id      = device_id

      super(id, name, updated_at, created_at)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, hash, limit, @@belongs_to)) if valid_key?(hash)
    end

    def self.short_name
      'timelapse'
    end

    private
    def self.valid_key?(hash)
      hash.keys.all? { |key| COLUMNS.push(*%i(id name updated_at created_at)).include? key }
    end

    def self.to_objects(timelapses)
      timelapses.map do |timelapse|
        new(
          id: timelapse['objectId'],
          url: timelapse['videoURL'],
          thumbnail_url: timelapse['thumbnail'] && timelapse['thumbnail']['url'],
          device_id: timelapse['device'] && timelapse['device']['objectId'],
          start_datetime: timelapse['beginningTime'] && timelapse['beginningTime']['iso'],
          end_datetime: timelapse['endTime'] && timelapse['endTime']['iso'],
          updated_at: timelapse['updatedAt'],
          created_at: timelapse['createdAt']
        )
      end
    end
  end
end
