module KakaxiParse
  class User < BaseModel
    attr_accessor :email, :password
    @@related_to = [Role]

    def initialize(
          id: nil,
          name: nil,
          email: nil,
          password: nil,
          updated_at: nil,
          created_at: nil
          )
      @email = email
      @password = password
      super(id, name, updated_at, created_at)
    end

    def self.where(hash, limit: DEFUALT_LIMIT)
      to_objects(super(self, name_to_nickname(hash), limit, [], @@related_to)) if valid_key?(hash)
    end

    def self.name
      '_User'
    end

    def self.short_name
      'farmer'
    end

    def self.reference_name
      'users'
    end

    def sign_in
      API.new('login', params: {username: email, password: password}).get
    end

    def save
      id ? false : super(self, { nickname: name, email: email, password: password, username: email })
    end

    def update(params)
      original_params = build_original_params(params)
      if super(self, User.name_to_nickname(params))
        original_params.each { |key, value| self.send("#{key}=", value) }
        true
      else
        false
      end
    end

    def farms
      Farm.where(farmer_id: id)
    end

    private
    def self.name_to_nickname(params)
      replace_key(params, :name, :nickname)
    end

    def self.valid_key?(hash)
      hash.keys.all? { |key| %i(id name email password updated_at created_at role_id).include? key }
    end

    def self.to_objects(users)
      users.map do |user|
        new(
          id: user['objectId'],
          name: user['nickname'],
          email: user['email'],
          updated_at: user['updatedAt'],
          created_at: user['createdAt']
        )
      end
    end
  end
end
